from typing import List
from abc import ABC, abstractmethod

from bpy.types import (
    Object, 
    Material,
    ShaderNodeGroup, 
)

from mesh_storage import MeshStorage
from serializer import SLink, SInput
from pyedm_platform_selector import pyedm


# # не наследуемся от этого класса.
# class MaterialWrap:
#     # спускаем материал и Textures, Values
#     # когда wrapper будет создаваться он будет из фабриkи брать класс, из него будет брать класс Textures и Values
#     def __init__(self, bpy_material: Material, node_group_type: str, node_group: ShaderNodeGroup) -> None:
#         self.material: Material = bpy_material
#         self.name: str = bpy_material.name
#         self.node_group_type = node_group_type
#         self.node_group = node_group #get_edm_node_group(bpy_material, self.node_group_type)
#         self.valid = not self.node_group == None
        
#         self.textures: material.Textures = None
#         self.values: material.Values = None
#         self.apply_group(self.node_group, material.Textures, material.Values)        
#         if not self.valid:
#             return
#         self.update_linked_resources()

#     def is_valid(self):
#         return self.valid
    
#     def apply_group(self):
#         self.valid = not self.node_group == None
#         if not self.valid:
#             return 

#         for attr in self.textures.__dict__.keys():
#             getattr(self.textures, attr).apply(self.node_group)

#         for attr in self.values.__dict__.keys():
#             getattr(self.values, attr).apply(self.node_group)
        
#     def update_linked_resources(self):
#         for i in self.textures.__dict__.keys():
#             getattr(self.textures, i).update()

#         for i in self.values.__dict__.keys():
#             getattr(self.values, i).update()

## base material class 
class IMaterial(ABC):
    """
    Common abstract class for all types of materials.
    """

    ## Material name. Now supported 4 materials: Deck, Default, Fake_Spot, Fake_Omni
    name: str = NotImplemented
    ## Relative path to .pickle file of serialised material.
    description_file_name: str = NotImplemented
    ## name of node group
    node_group_name: str = NotImplemented
 
    @abstractmethod
    def build_blocks(self, obj: Object, storage: MeshStorage) -> pyedm.IRenderNode:
        """
        Create render node that can be imported to edm. 
        Once 'pyedm.IRenderNode' was created, you need to set control node to it and add render node to model. 
        """
        pass

    #TODO-270 - add description for methods  
    @classmethod
    @abstractmethod
    def process_links(cls, links: List[SLink], version: int, group_node_type_name: str)-> List[SLink]:
        pass

    @classmethod
    @abstractmethod
    def restore_defaults(cls, old_sockest: List[SInput], 
                         new_node_group: ShaderNodeGroup, old_version: int, material_name: str) -> None:
        pass
